import _extends from "@babel/runtime/helpers/esm/extends";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { forwardRef, useImperativeHandle, useRef, useState } from "react";
import PropTypes from "prop-types";
import { useReactoForm } from "reacto-form";
import { uniqueId } from "lodash";
import { withComponents } from "@reactioncommerce/components-context";
import { CustomPropTypes } from "../../utils";
/**
 * Convert the form document to the object structure
 * expected by `PaymentsCheckoutAction`
 * @param {Object} Form object
 * @returns {Object} Transformed object
 */

function buildResult(_ref) {
  var amount = _ref.amount,
      _ref$fullName = _ref.fullName,
      fullName = _ref$fullName === void 0 ? null : _ref$fullName;
  var floatAmount = amount ? parseFloat(amount) : null;
  if (isNaN(floatAmount)) floatAmount = null;
  return {
    amount: floatAmount,
    data: {
      fullName: fullName
    },
    displayName: fullName ? "IOU from ".concat(fullName) : null
  };
}
/**
 * @summary ExampleIOUPaymentForm component
 * @param {Object} props Props
 * @param {Object} ref Ref
 * @return {Object} React render
 */


function ExampleIOUPaymentForm(props, ref) {
  var lastDocRef = useRef();
  var isReadyRef = useRef();

  var _useState = useState(),
      _useState2 = _slicedToArray(_useState, 2),
      uniqueInstanceIdentifier = _useState2[0],
      setUniqueInstanceIdentifier = _useState2[1];

  if (!uniqueInstanceIdentifier) {
    setUniqueInstanceIdentifier(uniqueId("ExampleIOUPaymentForm"));
  }

  var className = props.className,
      _props$components = props.components,
      ErrorsBlock = _props$components.ErrorsBlock,
      Field = _props$components.Field,
      TextInput = _props$components.TextInput,
      isSaving = props.isSaving,
      _onChange = props.onChange,
      onReadyForSaveChange = props.onReadyForSaveChange,
      _onSubmit = props.onSubmit;

  var _useReactoForm = useReactoForm({
    isReadOnly: isSaving,
    onChange: function onChange(formData) {
      var resultDoc = buildResult(formData);
      var stringDoc = JSON.stringify(resultDoc);

      if (stringDoc !== lastDocRef.current) {
        _onChange(resultDoc);
      }

      lastDocRef.current = stringDoc;
      var isReady = !!formData.fullName;

      if (isReady !== isReadyRef.current) {
        onReadyForSaveChange(isReady);
      }

      isReadyRef.current = isReady;
    },
    onSubmit: function onSubmit(formData) {
      return _onSubmit(buildResult(formData));
    }
  }),
      getErrors = _useReactoForm.getErrors,
      getInputProps = _useReactoForm.getInputProps,
      submitForm = _useReactoForm.submitForm;

  useImperativeHandle(ref, function () {
    return {
      submit: function submit() {
        submitForm();
      }
    };
  });
  var fullNameInputId = "fullName_".concat(uniqueInstanceIdentifier);
  var amountInputId = "amount_".concat(uniqueInstanceIdentifier);
  return React.createElement("div", {
    className: className
  }, React.createElement(Field, {
    name: "fullName",
    errors: getErrors(["fullName"]),
    label: "Full name",
    labelFor: fullNameInputId
  }, React.createElement(TextInput, _extends({
    id: fullNameInputId
  }, getInputProps("fullName"))), React.createElement(ErrorsBlock, {
    errors: getErrors(["fullName"])
  })), React.createElement(Field, {
    name: "amount",
    errors: getErrors(["amount"]),
    label: "Amount (optional)",
    labelFor: amountInputId
  }, React.createElement(TextInput, _extends({
    id: amountInputId
  }, getInputProps("amount"))), React.createElement(ErrorsBlock, {
    errors: getErrors(["amount"])
  })));
} // There is currently some issue with combining hoist-non-react-statics (used by
// withComponents) with forwardRef. Until that's resolved, reassigning
// ExampleIOUPaymentForm to the wrapped component here, before setting the statics.

/* eslint-disable-next-line no-func-assign */


ExampleIOUPaymentForm = withComponents(forwardRef(ExampleIOUPaymentForm));
ExampleIOUPaymentForm.propTypes = {
  /**
   * You can provide a `className` prop that will be applied to the outermost DOM element
   * rendered by this component. We do not recommend using this for styling purposes, but
   * it can be useful as a selector in some situations.
   */
  className: PropTypes.string,

  /**
   * If you've set up a components context using
   * [@reactioncommerce/components-context](https://github.com/reactioncommerce/components-context)
   * (recommended), then this prop will come from there automatically. If you have not
   * set up a components context or you want to override one of the components in a
   * single spot, you can pass in the components prop directly.
   */
  components: PropTypes.shape({
    /**
     * Pass either the Reaction ErrorsBlock component or your own component that
     * accepts compatible props.
     */
    ErrorsBlock: CustomPropTypes.component.isRequired,

    /**
     * Pass either the Reaction Field component or your own component that
     * accepts compatible props.
     */
    Field: CustomPropTypes.component.isRequired,

    /**
     * Pass either the Reaction TextInput component or your own component that
     * accepts compatible props.
     */
    TextInput: CustomPropTypes.component.isRequired
  }),

  /**
   * Pass true while the input data is in the process of being saved.
   * While true, the form fields are disabled.
   */
  isSaving: PropTypes.bool,

  /**
   * Called as the form fields are changed
   */
  onChange: PropTypes.func,

  /**
   * When this action's input data switches between being
   * ready for saving and not ready for saving, this will
   * be called with `true` (ready) or `false`
   */
  onReadyForSaveChange: PropTypes.func,

  /**
   * Called with an object value when this component's `submit`
   * method is called. The object may have `data`, `displayName`,
   * and `amount` properties.
   */
  onSubmit: PropTypes.func
};
ExampleIOUPaymentForm.defaultProps = {
  isSaving: false,
  onChange: function onChange() {},
  onReadyForSaveChange: function onReadyForSaveChange() {},
  onSubmit: function onSubmit() {}
};
export default ExampleIOUPaymentForm;